package lcsb.mapviewer.api.overlay;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;

import lcsb.mapviewer.api.BaseRestImpl;
import lcsb.mapviewer.api.QueryException;
import lcsb.mapviewer.common.Configuration;
import lcsb.mapviewer.model.cache.FileEntry;
import lcsb.mapviewer.model.map.layout.InvalidColorSchemaException;
import lcsb.mapviewer.model.map.layout.Layout;
import lcsb.mapviewer.model.map.model.Model;
import lcsb.mapviewer.model.user.User;
import lcsb.mapviewer.persist.dao.map.LayoutDao;
import lcsb.mapviewer.services.SecurityException;
import lcsb.mapviewer.services.interfaces.ILayoutService;
import lcsb.mapviewer.services.interfaces.ILayoutService.CreateLayoutParams;
import lcsb.mapviewer.services.interfaces.IModelService;
import lcsb.mapviewer.services.interfaces.IUserService;
import lcsb.mapviewer.services.search.data.ElementIdentifier.ElementIdentifierType;
import lcsb.mapviewer.services.search.layout.LightLayoutAliasView;
import lcsb.mapviewer.services.search.layout.LightLayoutReactionView;
import lcsb.mapviewer.services.utils.data.ColorSchemaType;
import lcsb.mapviewer.services.view.AuthenticationToken;
import lcsb.mapviewer.services.view.LayoutView;

@Transactional(value = "txManager")
public class OverlayRestImpl extends BaseRestImpl {

	Logger								 logger	= Logger.getLogger(OverlayRestImpl.class);

	@Autowired
	private IUserService	 userService;

	@Autowired
	private IModelService	 modelService;

	@Autowired
	private ILayoutService layoutService;

	@Autowired
	private LayoutDao			 layoutDao;

	public List<LayoutView> getOverlayList(String token, String projectId) throws SecurityException, QueryException {
		AuthenticationToken authenticationToken = userService.getToken(token);
		Model model = modelService.getLastModelByProjectId(projectId, authenticationToken);
		if (model == null) {
			throw new QueryException("Project with given id doesn't exist");
		}
		return layoutService.getCustomLayouts(model, token);
	}

	/**
	 * @return the userService
	 * @see #userService
	 */
	public IUserService getUserService() {
		return userService;
	}

	/**
	 * @param userService
	 *          the userService to set
	 * @see #userService
	 */
	public void setUserService(IUserService userService) {
		this.userService = userService;
	}

	/**
	 * @return the layoutService
	 * @see #layoutService
	 */
	public ILayoutService getLayoutService() {
		return layoutService;
	}

	/**
	 * @param layoutService
	 *          the layoutService to set
	 * @see #layoutService
	 */
	public void setLayoutService(ILayoutService layoutService) {
		this.layoutService = layoutService;
	}

	/**
	 * @return the modelService
	 * @see #modelService
	 */
	public IModelService getModelService() {
		return modelService;
	}

	/**
	 * @param modelService
	 *          the modelService to set
	 * @see #modelService
	 */
	public void setModelService(IModelService modelService) {
		this.modelService = modelService;
	}

	public List<Map<String, Object>> getOverlayElements(String token, String projectId, int overlayId, String columns) throws QueryException, SecurityException {
		List<Map<String, Object>> result = new ArrayList<>();
		AuthenticationToken authenticationToken = userService.getToken(token);

		Model model = modelService.getLastModelByProjectId(projectId, authenticationToken);
		if (model == null) {
			throw new QueryException("Project with given id doesn't exist");
		}
		List<LightLayoutAliasView> speciesList = layoutService.getAliasesForLayout(model, overlayId, authenticationToken);
		for (LightLayoutAliasView lightLayoutAliasView : speciesList) {
			Map<String, Object> element = new HashMap<>();
			element.put("type", ElementIdentifierType.ALIAS);
			element.put("overlayContent", lightLayoutAliasView);
			result.add(element);
		}

		List<LightLayoutReactionView> reactions = layoutService.getReactionsForLayout(model, overlayId, authenticationToken);
		for (LightLayoutReactionView lightReactionView : reactions) {
			Map<String, Object> element = new HashMap<>();
			element.put("type", ElementIdentifierType.REACTION);
			element.put("overlayContent", lightReactionView);
			result.add(element);
		}
		return result;
	}

	public LayoutView getOverlayById(String token, String projectId, String overlayId) throws SecurityException, QueryException {
		AuthenticationToken authenticationToken = userService.getToken(token);
		Model model = modelService.getLastModelByProjectId(projectId, authenticationToken);
		if (model == null) {
			throw new QueryException("Project with given id doesn't exist");
		}
		return layoutService.getLayoutById(model, Integer.valueOf(overlayId), authenticationToken);
	}

	public FileEntry getOverlaySource(String token, String projectId, String overlayId) throws SecurityException, QueryException {
		AuthenticationToken authenticationToken = userService.getToken(token);
		Model model = modelService.getLastModelByProjectId(projectId, authenticationToken);
		if (model == null) {
			throw new QueryException("Project with given id doesn't exist");
		}
		try {
			Integer id = Integer.valueOf(overlayId);
			Layout layout = layoutService.getLayoutDataById(id, authenticationToken);
			if (layout == null) {
				throw new QueryException("Invalid overlay id");
			}
			// lazy initialization issue
			layout.getInputData().getFileContent();
			return layout.getInputData();
		} catch (NumberFormatException e) {
			throw new QueryException("Invalid overlay id");
		}
	}

	public Map<String, Object> updateOverlay(String token, String projectId, String overlayId, String name, String description)
			throws QueryException, SecurityException {
		AuthenticationToken authenticationToken = userService.getToken(token);
		Model model = modelService.getLastModelByProjectId(projectId, authenticationToken);
		if (model == null) {
			throw new QueryException("Project with given id doesn't exist");
		}
		try {
			Integer id = Integer.valueOf(overlayId);
			Layout layout = layoutService.getLayoutDataById(id, authenticationToken);
			if (layout == null) {
				throw new QueryException("Invalid overlay id");
			}
			layout.setDescription(description);
			layout.setTitle(name);
			layoutDao.update(layout);
		} catch (NumberFormatException e) {
			throw new QueryException("Invalid overlay id");
		}
		return okStatus();
	}

	public Map<String, Object> removeOverlay(String token, String projectId, String overlayId) throws QueryException, SecurityException, IOException {
		AuthenticationToken authenticationToken = userService.getToken(token);
		Model model = modelService.getLastModelByProjectId(projectId, authenticationToken);
		if (model == null) {
			throw new QueryException("Project with given id doesn't exist");
		}
		try {
			Integer id = Integer.valueOf(overlayId);
			LayoutView layout = layoutService.getLayoutById(model, id, authenticationToken);
			if (layout == null) {
				throw new QueryException("Invalid overlay id");
			}
			layoutService.removeLayout(layout, null);
		} catch (NumberFormatException e) {
			throw new QueryException("Invalid overlay id");
		}
		return okStatus();
	}

	/**
	 * @return the layoutDao
	 * @see #layoutDao
	 */
	public LayoutDao getLayoutDao() {
		return layoutDao;
	}

	/**
	 * @param layoutDao
	 *          the layoutDao to set
	 * @see #layoutDao
	 */
	public void setLayoutDao(LayoutDao layoutDao) {
		this.layoutDao = layoutDao;
	}

	public Map<String, Object> addOverlay(String token, String projectId, String name, String description, String content, String filename, String type)
			throws SecurityException, QueryException, IOException {
		AuthenticationToken authenticationToken = userService.getToken(token);
		User user = userService.getUserByToken(token);
		if (Configuration.ANONYMOUS_LOGIN.equals(user.getLogin())) {
			throw new SecurityException("You have no privileges to add layout");
		}
		Model model = modelService.getLastModelByProjectId(projectId, authenticationToken);
		if (model == null) {
			throw new QueryException("Project with given id doesn't exist");
		}
		ColorSchemaType colorSchemaType = ColorSchemaType.GENERIC;
		logger.debug("TYPE: \'" + type + "\'");
		if (type != null && !type.equals("")) {
			try {
				colorSchemaType = ColorSchemaType.valueOf(type);
			} catch (IllegalArgumentException e) {
				throw new QueryException("Invalid type of overlay: " + type, e);
			}
		}

		try {
			InputStream stream = new ByteArrayInputStream(content.getBytes(StandardCharsets.UTF_8));

			LayoutView layout = layoutService.createLayout(
					new CreateLayoutParams()
							.async(false).colorInputStream(stream).description(description).layoutFileName(filename).model(model).name(name).user(user)
							.colorSchemaType(colorSchemaType).directory("."));
			Map<String, Object> result = okStatus();
			result.put("overlayId", layout.getIdObject());
			return result;
		} catch (InvalidColorSchemaException e) {
			throw new QueryException(e.getMessage(), e);
		}

	}

}
