package lcsb.mapviewer.api.project;

import java.io.Serializable;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.apache.log4j.Logger;

import lcsb.mapviewer.model.Project;
import lcsb.mapviewer.model.map.OverviewImage;
import lcsb.mapviewer.model.map.OverviewImageLink;
import lcsb.mapviewer.model.map.OverviewLink;
import lcsb.mapviewer.model.map.model.ModelData;
import lcsb.mapviewer.services.view.AnnotationView;
import lcsb.mapviewer.services.view.OverviewImageView;
import lcsb.mapviewer.services.view.OverviewImageViewFactory;

public class ProjectMetaData implements Serializable {

	private static Logger						logger					 = Logger.getLogger(ProjectMetaData.class);

	/**
	 * 
	 */
	private static final long				serialVersionUID = 1L;

	/**
	 * Version of the project.
	 */
	private String									version;

	private AnnotationView					disease;

	private AnnotationView					organism;

	private Integer									idObject;

	/**
	 * Name of the project.
	 */
	private String									name;

	/**
	 * {@link lcsb.mapviewer.model.Project#projectId Project identifier}.
	 */
	private String									projectId;

	/**
	 * Description of the model.
	 */
	private String									description;

	private ModelMetaData						map;

	/**
	 * List of overview images attached to this model.
	 */
	private List<OverviewImageView>	overviewImageViews;

	/**
	 * Top level overview image.
	 */
	private OverviewImageView				topOverviewImage;

	/**
	 * Default constructor. Should be used only for deserialization.
	 */
	protected ProjectMetaData() {
	}

	public ProjectMetaData(Project project) {
		OverviewImageViewFactory factory = new OverviewImageViewFactory();
		ModelData model = null;
		if (project.getModels().size() > 0) {
			model = project.getModels().iterator().next();
		}

		this.setName(project.getName());
		this.setProjectId(project.getProjectId());
		this.setIdObject(project.getId());

		if (model != null) {
			this.setOverviewImageViews(factory.createList(model.getOverviewImages()));
			this.setVersion(model.getMapVersion());
			this.setDescription(model.getNotes());

			Set<OverviewImage> set = new HashSet<OverviewImage>();
			set.addAll(model.getOverviewImages());
			for (OverviewImage image : model.getOverviewImages()) {
				for (OverviewLink ol : image.getLinks()) {
					if (ol instanceof OverviewImageLink) {
						set.remove(((OverviewImageLink) ol).getLinkedOverviewImage());
					}
				}
			}
			if (set.size() > 0) {
				this.setTopOverviewImage(factory.create(set.iterator().next()));
			} else if (model.getOverviewImages().size() > 0) {
				logger.warn("Cannot determine top level image. Taking first one. " + model.getOverviewImages().get(0).getFilename());
				this.setTopOverviewImage(factory.create(model.getOverviewImages().get(0)));
			}
			this.setMap(new ModelMetaData(model));

		}
	}

	/**
	 * @return the version
	 * @see #version
	 */
	public String getVersion() {
		return version;
	}

	/**
	 * @param version
	 *          the version to set
	 * @see #version
	 */
	public void setVersion(String version) {
		this.version = version;
	}

	/**
	 * @return the description
	 * @see #description
	 */
	public String getDescription() {
		return description;
	}

	/**
	 * @param description
	 *          the description to set
	 * @see #description
	 */
	public void setDescription(String description) {
		this.description = description;
	}

	/**
	 * @return the name
	 * @see #name
	 */
	public String getName() {
		return name;
	}

	/**
	 * @param name
	 *          the name to set
	 * @see #name
	 */
	public void setName(String name) {
		this.name = name;
	}

	/**
	 * @return the overviewImageViews
	 * @see #overviewImageViews
	 */
	public List<OverviewImageView> getOverviewImageViews() {
		return overviewImageViews;
	}

	/**
	 * @param overviewImageViews
	 *          the overviewImageViews to set
	 * @see #overviewImageViews
	 */
	public void setOverviewImageViews(List<OverviewImageView> overviewImageViews) {
		this.overviewImageViews = overviewImageViews;
	}

	/**
	 * @return the topOverviewImage
	 * @see #topOverviewImage
	 */
	public OverviewImageView getTopOverviewImage() {
		return topOverviewImage;
	}

	/**
	 * @param topOverviewImage
	 *          the topOverviewImage to set
	 * @see #topOverviewImage
	 */
	public void setTopOverviewImage(OverviewImageView topOverviewImage) {
		this.topOverviewImage = topOverviewImage;
	}

	/**
	 * @return the projectId
	 * @see #projectId
	 */
	public String getProjectId() {
		return projectId;
	}

	/**
	 * @param projectId
	 *          the projectName to set
	 * @see #projectId
	 */
	public void setProjectId(String projectId) {
		this.projectId = projectId;
	}

	/**
	 * @return the map
	 * @see #map
	 */
	public ModelMetaData getMap() {
		return map;
	}

	/**
	 * @param map
	 *          the map to set
	 * @see #map
	 */
	public void setMap(ModelMetaData map) {
		this.map = map;
	}

	/**
	 * @return the idObject
	 * @see #idObject
	 */
	public Integer getIdObject() {
		return idObject;
	}

	/**
	 * @param idObject
	 *          the idObject to set
	 * @see #idObject
	 */
	public void setIdObject(Integer idObject) {
		this.idObject = idObject;
	}

	/**
	 * @return the disease
	 * @see #disease
	 */
	public AnnotationView getDisease() {
		return disease;
	}

	/**
	 * @param disease
	 *          the disease to set
	 * @see #disease
	 */
	public void setDisease(AnnotationView disease) {
		this.disease = disease;
	}

	/**
	 * @return the organism
	 * @see #organism
	 */
	public AnnotationView getOrganism() {
		return organism;
	}

	/**
	 * @param organism
	 *          the organism to set
	 * @see #organism
	 */
	public void setOrganism(AnnotationView organism) {
		this.organism = organism;
	}

}
