package lcsb.mapviewer.api.project;

import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.commons.lang3.StringUtils;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.RequestParam;

import lcsb.mapviewer.common.exception.InvalidStateException;
import lcsb.mapviewer.model.Project;
import lcsb.mapviewer.model.map.model.Model;
import lcsb.mapviewer.model.map.reaction.Modifier;
import lcsb.mapviewer.model.map.reaction.Product;
import lcsb.mapviewer.model.map.reaction.Reactant;
import lcsb.mapviewer.model.map.reaction.Reaction;
import lcsb.mapviewer.model.map.species.Element;
import lcsb.mapviewer.services.SecurityException;
import lcsb.mapviewer.services.UserAccessException;
import lcsb.mapviewer.services.interfaces.IModelService;
import lcsb.mapviewer.services.interfaces.IProjectService;
import lcsb.mapviewer.services.interfaces.ISearchService;
import lcsb.mapviewer.services.interfaces.IUserService;
import lcsb.mapviewer.services.search.data.ElementIdentifier.ElementIdentifierType;
import lcsb.mapviewer.services.search.data.LightReactionView;
import lcsb.mapviewer.services.view.AnnotationViewFactory;

@Transactional(value = "txManager")
public class ProjectRestImpl {
	Logger									logger = Logger.getLogger(ProjectRestImpl.class);

	@Autowired
	private IUserService		userService;

	@Autowired
	private IProjectService	projectService;

	@Autowired
	private IModelService		modelService;

	@Autowired
	private ISearchService	searchService;

	@Autowired
	AnnotationViewFactory		annotationViewFactory;

	public ProjectMetaData getMetaData(@RequestParam(value = "projectId") String projectId, @RequestParam(value = "token") String token)
			throws SecurityException {

		Project project = projectService.getProjectByProjectId(projectId, userService.getToken(token));
		ProjectMetaData result = new ProjectMetaData(project);
		if (project.getOrganism() != null) {
			result.setOrganism(annotationViewFactory.create(project.getOrganism()));
		}
		if (project.getDisease() != null) {
			result.setDisease(annotationViewFactory.create(project.getDisease()));
		}
		return result;
	}

	/**
	 * @return the userService
	 * @see #userService
	 */
	public IUserService getUserService() {
		return userService;
	}

	/**
	 * @param userService
	 *          the userService to set
	 * @see #userService
	 */
	public void setUserService(IUserService userService) {
		this.userService = userService;
	}

	public List<Map<String, Object>> getElements(String projectId, String id, String columns, String token) throws UserAccessException, SecurityException {
		Model model = modelService.getLastModelByProjectId(projectId, userService.getToken(token));
		Set<Integer> ids = new HashSet<>();
		if (!id.equals("")) {
			for (String str : id.split(",")) {
				ids.add(Integer.valueOf(str));
			}
		}
		Set<String> columnsSet = createElementColumnSet(columns);

		List<Map<String, Object>> result = new ArrayList<>();

		List<Model> models = new ArrayList<>();
		models.addAll(model.getSubmodels());
		models.add(model);
		for (Model model2 : models) {
			for (Element element : model2.getElements()) {
				if (ids.size() == 0 || ids.contains(element.getId())) {
					result.add(preparedElement(element, columnsSet));
				}
			}
		}

		return result;
	}

	public List<Map<String, Object>> getClosestElementsByCoordinates(String projectId, String modelId, String token, Point2D coordinates, Integer count)
			throws UserAccessException, SecurityException {
		List<Map<String, Object>> resultMap = new ArrayList<>();

		Model model = modelService.getLastModelByProjectId(projectId, userService.getToken(token));

		Model submodel = model.getSubmodelById(modelId);

		List<Object> elements = searchService.getClosestElements(submodel, coordinates, count);
		for (Object object : elements) {
			Map<String, Object> result = createMinifiedSearchResult(object);
			resultMap.add(result);
		}
		return resultMap;
	}

	private Map<String, Object> createMinifiedSearchResult(Object object) {
		Map<String, Object> result = new HashMap<>();
		if (object instanceof Element) {
			result.put("type", ElementIdentifierType.ALIAS);
			Element element = (Element) object;
			result.put("id", element.getId());
			result.put("modelId", element.getModel().getId());
		} else if (object instanceof Reaction) {
			result.put("type", ElementIdentifierType.REACTION);
			Reaction element = (Reaction) object;
			result.put("id", element.getId());
			result.put("modelId", element.getModel().getId());

		} else {
			throw new InvalidStateException("Unknown type of result: " + object.getClass());
		}
		return result;
	};

	public List<Map<String, Object>> getReactions(String projectId, String id, String columns, String token) throws UserAccessException, SecurityException {
		Model model = modelService.getLastModelByProjectId(projectId, userService.getToken(token));
		Set<Integer> ids = new HashSet<>();
		if (!id.equals("")) {
			for (String str : id.split(",")) {
				ids.add(Integer.valueOf(str));
			}
		}
		Set<String> columnsSet = createReactionColumnSet(columns);

		List<Map<String, Object>> result = new ArrayList<>();

		List<Model> models = new ArrayList<>();
		models.addAll(model.getSubmodels());
		models.add(model);
		for (Model model2 : models) {
			for (Reaction reaction : model2.getReactions()) {
				if (ids.size() == 0 || ids.contains(reaction.getId())) {
					result.add(preparedReaction(reaction, columnsSet));
				}
			}
		}

		return result;
	}

	private Map<String, Object> preparedReaction(Reaction reaction, Set<String> columnsSet) {
		Map<String, Object> result = new HashMap<>();
		for (String string : columnsSet) {
			String column = string.toLowerCase();
			Object value = null;
			if (column.equals("id") || column.equals("idobject")) {
				value = reaction.getId();
			} else if (column.equals("modelid")) {
				value = reaction.getModelData().getId();
			} else if (column.equals("reactionid")) {
				value = reaction.getIdReaction();
			} else if (column.equals("name")) {
				value = reaction.getName();
			} else if (column.equals("centerpoint")) {
				value = reaction.getCenterPoint();
			} else if (column.equals("products")) {
				List<Integer> ids = new ArrayList<>();
				for (Product product : reaction.getProducts()) {
					ids.add(product.getElement().getId());
				}
				value = StringUtils.join(ids, ",");
			} else if (column.equals("reactants")) {
				List<Integer> ids = new ArrayList<>();
				for (Reactant reactant : reaction.getReactants()) {
					ids.add(reactant.getElement().getId());
				}
				value = StringUtils.join(ids, ",");
			} else if (column.equals("modifiers")) {
				List<Integer> ids = new ArrayList<>();
				for (Modifier product : reaction.getModifiers()) {
					ids.add(product.getElement().getId());
				}
				value = StringUtils.join(ids, ",");
			} else if (column.equals("type")) {
				value = reaction.getStringType();
			} else if (column.equals("lines")) {
				value = new LightReactionView(reaction).getLines();
			} else {
				value = "Unknown column";
			}
			result.put(string, value);
		}
		return result;
	}

	private Map<String, Object> preparedElement(Element element, Set<String> columnsSet) {
		Map<String, Object> result = new HashMap<>();
		for (String string : columnsSet) {
			String column = string.toLowerCase();
			Object value = null;
			if (column.equals("id") || column.equals("idobject")) {
				value = element.getId();
			} else if (column.equals("modelid")) {
				value = element.getModelData().getId();
			} else if (column.equals("name")) {
				value = element.getName();
			} else if (column.equals("type")) {
				value = element.getStringType();
			} else if (column.equals("symbol")) {
				value = element.getSymbol();
			} else if (column.equals("fullname")) {
				value = element.getFullName();
			} else if (column.equals("abbreviation")) {
				value = element.getAbbreviation();
			} else if (column.equals("references")) {
				value = annotationViewFactory.createList(element.getMiriamData());
			} else if (column.equals("synonyms")) {
				value = element.getSynonyms();
			} else if (column.equals("formula")) {
				value = element.getFormula();
			} else if (column.equals("description")) {
				value = element.getNotes();
			} else if (column.equals("formersymbols")) {
				value = element.getFormerSymbols();
			} else if (column.equals("bounds")) {
				value = new Rectangle2D.Double(element.getX(), element.getY(), element.getWidth(), element.getHeight());
			} else {
				value = "Unknown column";
			}
			result.put(string, value);
		}
		return result;
	}

	private Set<String> createElementColumnSet(String columns) {
		Set<String> columnsSet = new HashSet<>();
		if (columns.equals("")) {
			columnsSet.add("id");
			columnsSet.add("modelId");
			columnsSet.add("name");
			columnsSet.add("type");
			columnsSet.add("description");
			columnsSet.add("type");
			columnsSet.add("symbol");
			columnsSet.add("fullName");
			columnsSet.add("abbreviation");
			columnsSet.add("formula");
			columnsSet.add("name");
			columnsSet.add("synonyms");
			columnsSet.add("formerSymbols");
			columnsSet.add("references");
			columnsSet.add("bounds");
		} else {
			for (String str : columns.split(",")) {
				columnsSet.add(str);
			}
		}
		return columnsSet;
	}

	private Set<String> createReactionColumnSet(String columns) {
		Set<String> columnsSet = new HashSet<>();
		if (columns.equals("")) {
			columnsSet.add("id");
			columnsSet.add("reactionId");
			columnsSet.add("modelId");
			columnsSet.add("type");
			columnsSet.add("lines");
			columnsSet.add("centerPoint");
			columnsSet.add("products");
			columnsSet.add("reactants");
			columnsSet.add("modifiers");
		} else {
			for (String str : columns.split(",")) {
				columnsSet.add(str);
			}
		}
		return columnsSet;
	}

	/**
	 * @return the modelService
	 * @see #modelService
	 */
	public IModelService getModelService() {
		return modelService;
	}

	/**
	 * @param modelService
	 *          the modelService to set
	 * @see #modelService
	 */
	public void setModelService(IModelService modelService) {
		this.modelService = modelService;
	}

	/**
	 * @return the searchService
	 * @see #searchService
	 */
	public ISearchService getSearchService() {
		return searchService;
	}

	/**
	 * @param searchService
	 *          the searchService to set
	 * @see #searchService
	 */
	public void setSearchService(ISearchService searchService) {
		this.searchService = searchService;
	}

	public List<Map<String, Object>> getElementsByQuery(String projectId, String token, String query, Integer maxElements, String perfectMatch)
			throws SecurityException {
		List<Map<String, Object>> resultMap = new ArrayList<>();

		Model model = modelService.getLastModelByProjectId(projectId, userService.getToken(token));

		Integer limit = Integer.valueOf(maxElements);
		boolean match = perfectMatch.equals("true");
		List<Object> elements = searchService.searchByQuery(model, query, limit, match);
		for (Object object : elements) {
			Map<String, Object> result = createMinifiedSearchResult(object);
			resultMap.add(result);
		}
		return resultMap;
	}

	public String[] getSuggestedQueryList(String projectId, String token) throws SecurityException {
		Model model = modelService.getLastModelByProjectId(projectId, userService.getToken(token));
		return searchService.getSuggestedQueryList(model);
	}

}
